require 'fanli/version'
require 'fanli/base'

# Fanli
module Fanli
  @registry = {}

  def self.on(ev, *listeners)
    if listeners.size  == 1
      add_listener(@registry, ev.to_sym, listeners.first)
    else
      listeners.each do |ln|
        add_listener(@registry, ev.to_sym, ln)
      end
    end
  end

  def self.trigger(ev, with:)
    apply_callback("on_#{ev}", with, @registry[ev.to_sym])
  end

  def self.reset
    @registry = {}
  end

  def self.add_listener(registry, ev, listener)
    queue = registry[ev]
    if queue
      queue << listener
    else
      registry[ev] = [listener]
    end
  end

  def self.apply_callback(callback, data, objs)
    return if objs.nil?
    objs.map do |obj|
      if obj.respond_to?(callback)
        obj.send(callback, data)
      elsif obj.respond_to?(:call)
        obj.call(data)
      else
        next
      end
    end
  end
end
